from marshmallow import SchemaOpts, pre_dump

__all__ = ['ExpandableSchemaMixin', 'ExpandableViewMixin', 'ExpandableOpts']


class ExpandableOpts(SchemaOpts):
    """
    Adds support for expandable_fields to Class Meta. `expandable_fields` should be a
    dict of key = field name to be added to serialized results and val = a marshmallow Nested field.

    Example:
        expandable_fields = dict(paymentmethod=fields.Nested(PaymentMethodSchema, required=False))
    """

    def __init__(self, meta):
        super(ExpandableOpts, self).__init__(meta)
        self.expandable_fields = getattr(meta, 'expandable_fields', dict())


class ExpandableSchemaMixin:
    """
    Supports optionally expandable fields based on QUERY_KEY query strings included in the
    request's url.
    """

    OPTIONS_CLASS = ExpandableOpts
    QUERY_KEY = 'expand'

    @pre_dump
    def update_expandables(self, data):
        request = self.context.get('request')

        if request:
            requested_expands = list(val for key, val in request.params.items() if key == self.QUERY_KEY)
            available_expands = self.opts.expandable_fields.keys()

            for field in requested_expands:
                if field in available_expands:
                    self.declared_fields[field] = self.opts.expandable_fields[field]

        return data


class ExpandableViewMixin:
    """
    Optionally used to allow more fine grained control over the query used to pull data.
    `expandable_fields` should be a dict of key = the field name that is expandable
    and val = is a dict with the following keys:

    column (required): The table column that represents the relationship to be expanded.
    outerjoin (optional): Whether or not to use an outer join when joining the relationship.
    options (optional): kwargs passed to the constructed queries' options method.

    """

    expandable_fields = None

    def get_query(self):
        """
        If your query is more complicated than what is supported below, override this method.
        Don't forget to call super though.
        """

        query = super(ExpandableViewMixin, self).get_query()
        expandable_fields = getattr(self, 'expandable_fields', [])
        query_key = self.schema_class.QUERY_KEY

        if expandable_fields:
            requested_expands = list(val for key, val in self.request.params.items() if key == query_key)

            if requested_expands:
                available_expands = self.expandable_fields.keys()

                for name in requested_expands:
                    if name in available_expands:
                        field = self.expandable_fields[name]

                        if field.get('outerjoin', False):
                            query = query.outerjoin(field['column'])
                        else:
                            query = query.join(field['column'])

                        # Apply optional options
                        options = field.get('options')

                        if options:
                            query = query.options(*options)

        return query



class BasePagination:

    def paginate_query(self, queryset, request):
        raise NotImplementedError('paginate_query() must be implemented.')  # pragma: no cover

    def get_paginated_response(self, data):
        raise NotImplementedError('get_paginated_response() must be implemented.')  # pragma: no cover



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import sys

from io import open
from setuptools import setup

name = 'pyramid-restful-framework'
package = 'pyramid_restful'
description = 'RESTful API framework for Pyramid.'
url = 'https://github.com/danpoland/pyramid-restful-framework'
author = 'Daniel Poland'
author_email = 'danpoland84@gmail.com'
install_requires = [
    'six',
    'pyramid',
    'sqlalchemy',
    'marshmallow',
]

tests_require = [
    'WebTest >= 1.3.1',  # py3 compat
    'pytest'
]

here = os.path.abspath(os.path.dirname(__file__))

try:
    with open(os.path.join(here, 'README.md')) as f:
        README = f.read()
except IOError:
    README = ''


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """

    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a package themselves.
    """

    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []

    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])

    return {package: filepaths}


version = get_version(package)

if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep wheel"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()

    os.system("python setup.py sdist upload")
    os.system("python setup.py bdist_wheel upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a {0} -m 'version {0}'".format(version))
    print("  git push --tags")
    sys.exit()

setup(
    name=name,
    version=version,
    url=url,
    license='BSD',
    description=description,
    long_description=README,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires,
    setup_requires=['pytest-runner'],
    tests_require=tests_require,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Pyramid',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



class BasePermission:
    """
    All permission classes should inherit from this class.

    Override message to customize the message associated with the exception.
    """

    message = None

    def has_permission(self, request, view):
        """
        Return `True` if permission is granted else false.
        """

        return True

    def has_object_permission(self, request, view, obj):
        """
        Return `True` if permission is granted else false.
        """

        return True
